import Http from './services/Http';
import { AxiosInstance, AxiosRequestConfig } from 'axios';
/**
 * Api class
 *
 * Base class to manage calls to API
 */
export default class ApiCore {
    /**
     * The loading state of the service
     */
    loading: boolean;
    /**
     * Any error that was returned by the last call
     */
    error: any;
    /**
     * The Http service which makes the API call
     */
    http: Http;
    /**
     * The API constructor function
     *
     * @param   axios     An Axios instance
     */
    constructor(axios: AxiosInstance);
    /**
     * Use one of the built-in plugins
     *
     * @param   name      The name of the plugin
     * @param   params    Any options the plugin needs
     * @returns {this}
     */
    use(name: any, ...params: any[]): this;
    /**
     * Query the API using a request config object
     *
     * @param   config    The HTTP method to make the call
     * @param   data      Any optional data to pass to the endpoints
     * @returns
     */
    request(config: AxiosRequestConfig, data?: any): Promise<any>;
    /**
     * Call the API via HTTP GET
     *
     * @param   url       The API URL to call
     * @param   data      Any optional data to pass to the endpoint
     * @param   options   An optional hash of options to merge into the Axios config
     * @returns
     */
    get(url: string, data?: any, options?: any): Promise<any>;
    /**
     * Call the API via HTTP POST
     *
     * @param   url       The API URL to call
     * @param   data      Any optional data to pass to the endpoint
     * @param   options   An optional hash of options to merge into the Axios config
     * @returns
     */
    post(url: string, data?: any, options?: any): Promise<any>;
    /**
     * Add a callback to fire when any call completes successfully
     *
     * @param   callback  The function to fire
     * @returns
     */
    done(callback: any): this;
    /**
     * Add a callback to fire when any call fails to complete
     *
     * @param   callback  The function to fire
     * @returns
     */
    fail(callback: any): this;
}
