import ApiCore from './ApiCore';
import ActionMap from './services/ActionMap';
/**
 * Group class
 *
 * Manages related URLs of an API
 */
export default class ApiGroup extends ApiCore {
    /**
     * A map of API endpoints
     */
    actions: ActionMap;
    /**
     * Endpoint constructor
     *
     * @param   axios       An Axios instance
     * @param   actions     An optional hash of URLs
     */
    constructor(axios: any, actions?: any);
    /**
     * Add a new action
     *
     * @param   action      The name of the action to add
     * @param   config      The url, and optionally method and url, of the API endpoint
     * @param   callback?   An optional handler function to fire on a successful call
     */
    add(action: string, config: string, callback?: Function): any;
    /**
     * Add a new action
     *
     * @param   action      The name of the action to add
     * @param   config      An object conforming to the AxiosRequestConfig interface
     * @param   callback?   An optional handler function to fire on a successful call
     */
    add(action: string, config: any, callback?: Function): any;
    /**
     * Call a specific action
     *
     * @param   name      The name of the action to execute
     * @param   data      An optional hash of data to send to the server
     * @param   options   An optional hash of options to merge into the Axios config
     * @returns {Promise<any>}
     */
    call(name: string, data?: any, options?: any): Promise<any>;
    /**
     * Add a callback to fire when any call completes successfully
     *
     * @param   name      The name or names of actions to bind to
     * @param   callback  The function to fire
     * @returns this
     */
    when(name: string, callback: Function): this;
}
